@@ -4,29 +4,20 @@
 Utilities for performing repetitive tasks over potentially large sets
 of data over an extended period of time.
 """
-MANHOLE_PORT = 6022
+
 import weakref, datetime, os, sys
 
 from zope.interface import implements
 
 from twisted.python import reflect, failure, log, procutils, util, runtime
 from twisted.internet import task, defer, reactor, error, protocol
-from twisted.application import internet, service
-from twisted.cred import checkers, portal
-
-from twisted.conch.manhole import ColoredManhole
-from twisted.conch.insults import insults
-try:
-    from twisted.conch import manhole_ssh
-except ImportError:
-    manhole_ssh = None
-from twisted.conch import interfaces as iconch
+from twisted.application import service
 
 from epsilon import extime, process, cooperator, modal, juice
 
 from axiom import iaxiom, errors as eaxiom, item, attributes
 from axiom.scheduler import Scheduler, SubScheduler
-from axiom.upgrade import registerUpgrader
+from axiom.upgrade import registerUpgrader, registerDeletionUpgrader
 from axiom.dependency import installOn
 
 VERBOSE = False
@@ -274,14 +265,6 @@ class _ReliableListener(item.Item):
 
 class _BatchProcessorMixin:
 
-    def installed(self):
-        # XXX This is kind of suboptimal because it circumvents the usual
-        # dependency mechanism. But see #1408.
-
-        sch = iaxiom.IScheduler(self.store, None)
-        if sch is None:
-            installOn(SubScheduler(store=self.store), self.store)
-
     def step(self, style=iaxiom.LOCAL, skip=()):
         now = extime.Time()
         first = True
@@ -553,22 +536,24 @@ def _childProcTerminated(self, err):
 
 
 class ProcessController(object):
-    """Stateful class which tracks a Juice connection to a child process.
+    """
+    Stateful class which tracks a Juice connection to a child process.
 
     Communication occurs over stdin and stdout of the child process.  The
     process is launched and restarted as necessary.  Failures due to the child
     process terminating, either unilaterally of by request, are represented as
     a transient exception class,
 
-    Mode is one of
-      'stopped'       (no process running or starting)
-      'starting'      (process begun but not ready for requests)
-      'ready'         (process ready for requests)
-      'stopping'      (process being torn down)
-      'waiting_ready' (process beginning but will be shut down
-                          as soon as it starts up)
+    Mode is one of::
 
-    Transitions are as follows
+      - 'stopped'       (no process running or starting)
+      - 'starting'      (process begun but not ready for requests)
+      - 'ready'         (process ready for requests)
+      - 'stopping'      (process being torn down)
+      - 'waiting_ready' (process beginning but will be shut down
+                         as soon as it starts up)
+
+    Transitions are as follows::
 
        getProcess:
            stopped -> starting:
@@ -933,7 +918,6 @@ class BatchProcessingControllerService(service.Service):
             self._setStore, self._restartProcess,
             logdir.child("batch.log").path,
             rundir.child("batch.pid").path)
-        return self.batchController.getProcess()
 
 
     def _setStore(self):
@@ -1038,18 +1022,6 @@ class BatchProcessingProtocol(JuiceChild):
         self.pollCall = task.LoopingCall(self._pollSubStores)
         self.pollCall.start(10.0)
 
-        if manhole_ssh is not None:
-            rlm = portal.IRealm(self.siteStore, None)
-            if rlm is not None:
-                #don't wanna freak out if there's no userbase installed
-                chk = checkers.ICredentialsChecker(self.siteStore, None)
-                ptl = portal.Portal(rlm, [chk])
-                f = manhole_ssh.ConchFactory(ptl)
-                try:
-                    csvc = internet.TCPServer(MANHOLE_PORT, f)
-                    csvc.setServiceParent(self.service)
-                except error.CannotListenError:
-                    pass
         return {}
 
     command_SET_STORE.command = SetStore
@@ -1138,12 +1110,6 @@ class BatchProcessingService(service.Service):
         return defer.maybeDeferred(getattr(self.store.getItemByID(storeID), methodName))
 
 
-    def deferLater(self, n):
-        d = defer.Deferred()
-        reactor.callLater(n, d.callback, None)
-        return d
-
-
     def items(self):
         return self.store.powerupsFor(iaxiom.IBatchProcessor)
 
@@ -1152,8 +1118,8 @@ class BatchProcessingService(service.Service):
         """
         Run tasks until stopService is called.
         """
-        task = self.step()
-        for result, more in task:
+        work = self.step()
+        for result, more in work:
             yield result
             if not self.running:
                 break
@@ -1161,7 +1127,7 @@ class BatchProcessingService(service.Service):
                 delay = 0.1
             else:
                 delay = 10.0
-            yield self.deferLater(delay)
+            yield task.deferLater(reactor, delay, lambda: None)
 
 
     def step(self):
@@ -1212,36 +1178,14 @@ class BatchProcessingService(service.Service):
 
 
 
-if manhole_ssh is not None:
-    bases = (manhole_ssh.TerminalUser, manhole_ssh.TerminalSession)
-else:
-    bases = ()
-
 class BatchManholePowerup(item.Item):
-    installedOn = attributes.reference()
+    """
+    Previously, an L{IConchUser} powerup.  This class is only still defined for
+    schema compatibility.  Any instances of it will be deleted by an upgrader.
+    See #1001.
+    """
+    schemaVersion = 2
+    unused = attributes.integer(
+        doc="Satisfy Axiom requirement for at least one attribute")
 
-    original = attributes.inmemory()
-    transportFactory = attributes.inmemory()
-    chainedProtocolFactory = attributes.inmemory()
-    channelLookup = attributes.inmemory()
-    subsystemLookup = attributes.inmemory()
-    width = attributes.inmemory()
-    height = attributes.inmemory()
-    conn = attributes.inmemory()
-
-    powerupInterfaces = (iconch.IConchUser, iconch.ISession)
-
-    def activate(self):
-        if manhole_ssh is not None:
-            manhole_ssh.TerminalSession.__init__(self, self.store)
-            manhole_ssh.TerminalUser.__init__(self, self.store, None)
-
-
-    def chainedProtocolFactory(self):
-        return insults.ServerProtocol(
-            ColoredManhole,
-            None)
-
-
-# I don't even know what to say here. -exarkun
-BatchManholePowerup.__bases__ += bases
+registerDeletionUpgrader(BatchManholePowerup, 1, 2)
