@@ -23,7 +23,7 @@ from twisted.application.service import IService, IServiceCollection
 from epsilon.pending import PendingEvent
 from epsilon.cooperator import SchedulingService
 
-from axiom import _schema, attributes, upgrade, _fincache, iaxiom, errors, batch
+from axiom import _schema, attributes, upgrade, _fincache, iaxiom, errors
 from axiom import item
 from axiom._pysqlite2 import Connection
 
@@ -112,6 +112,14 @@ _noItem = object()              # tag for optional argument to getItemByID
 
 
 def storeServiceSpecialCase(st, pups):
+    """
+    Adapt a store to L{IServiceCollection}.
+
+    @param st: The L{Store} to adapt.
+    @param pups: A list of L{IServiceCollection} powerups on C{st}.
+
+    @return: An L{IServiceCollection} which has all of C{pups} as children.
+    """
     if st.parent is not None:
         # If for some bizarre reason we're starting a substore's service, let's
         # just assume that its parent is running its upgraders, rather than
@@ -128,6 +136,7 @@ def storeServiceSpecialCase(st, pups):
     st._upgradeService.setServiceParent(collection)
 
     if st.dbdir is not None:
+        from axiom import batch
         batcher = batch.BatchProcessingControllerService(st)
         batcher.setServiceParent(collection)
 
@@ -215,19 +224,18 @@ class BaseQuery:
 
     def explain(self):
         """
-        A debugging API, exposing SQLite's 'EXPLAIN' statement.
+        A debugging API, exposing SQLite's I{EXPLAIN} statement.
 
         While this is not a private method, you also probably don't have any
-        use for it unless you understand this page very well:
-
-            http://www.sqlite.org/opcode.html
+        use for it unless you understand U{SQLite
+        opcodes<http://www.sqlite.org/opcode.html>} very well.
 
         Once you do, it can be handy to call this interactively to get a sense
         of the complexity of a query.
 
         @return: a list, the first element of which is a L{str} (the SQL
         statement which will be run), and the remainder of which is 3-tuples
-        resulting from the 'EXPLAIN' of that statement.
+        resulting from the I{EXPLAIN} of that statement.
         """
         return ([self._sqlAndArgs('SELECT', self._queryTarget)[0]] +
                 self._runQuery('EXPLAIN SELECT', self._queryTarget))
@@ -389,7 +397,7 @@ class BaseQuery:
                 => [1, 2, 3]
 
         You can also use distinct queries to eliminate duplicate results from
-        joining two Item types together in a query, like so:
+        joining two Item types together in a query, like so::
 
             x = X(store=s, value=1, name=u'hello')
             Y(store=s, other=x, ident=u'a')
@@ -674,7 +682,7 @@ class MultipleItemQuery(BaseQuery):
         # self.tableClass is a tuple of Item classes.
         for tableClass in self.tableClass:
 
-            schema = tuple(tableClass.getSchema())
+            schema = tableClass.getSchema()
 
             # The extra 1 is oid
             self.schemaLengths.append(len(schema) + 1)
@@ -947,6 +955,45 @@ class AttributeQuery(BaseQuery):
         return self.attribute.outfilter(dbval, _FakeItemForFilter(self.store))
 
 
+def _storeBatchServiceSpecialCase(*args, **kwargs):
+    """
+    Trivial wrapper around L{batch.storeBatchServiceSpecialCase} to delay the
+    import of axiom.batch, which imports the reactor, which we do not want as a
+    side-effect of importing L{axiom.store} (as this would preclude selecting a
+    reactor after importing this module; see #2864).
+    """
+    from axiom import batch
+    return batch.storeBatchServiceSpecialCase(*args, **kwargs)
+
+
+def _schedulerServiceSpecialCase(empowered, pups):
+    """
+    This function creates (or returns a previously created) L{IScheduler}
+    powerup.
+
+    If L{IScheduler} powerups were found on C{empowered}, the first of those
+    is given priority.  Otherwise, a site L{Store} or a user L{Store} will
+    have any pre-existing L{IScheduler} powerup associated with them (on the
+    hackish cache attribute C{_schedulerService}) returned, or a new one
+    created if none exists already.
+    """
+    from axiom.scheduler import _SiteScheduler, _UserScheduler
+
+    # Give precedence to anything found in the store
+    for pup in pups:
+        return pup
+    # If the empowered is a store, construct a scheduler for it.
+    if isinstance(empowered, Store):
+        if getattr(empowered, '_schedulerService', None) is None:
+            if empowered.parent is None:
+                sched = _SiteScheduler(empowered)
+            else:
+                sched = _UserScheduler(empowered)
+            sched.setServiceParent(IService(empowered))
+            empowered._schedulerService = sched
+        return empowered._schedulerService
+    return None
+
 
 class Store(Empowered):
     """
@@ -954,7 +1001,7 @@ class Store(Empowered):
 
     Store an item in me by setting its 'store' attribute to be me.
 
-    I can be created one of two ways:
+    I can be created one of two ways::
 
         Store()                      # Create an in-memory database
 
@@ -980,7 +1027,8 @@ class Store(Empowered):
     aggregateInterfaces = {
         IService: storeServiceSpecialCase,
         IServiceCollection: storeServiceSpecialCase,
-        iaxiom.IBatchService: batch.storeBatchServiceSpecialCase}
+        iaxiom.IBatchService: _storeBatchServiceSpecialCase,
+        iaxiom.IScheduler: _schedulerServiceSpecialCase}
 
     implements(iaxiom.IBeneficiary)
 
@@ -1162,6 +1210,8 @@ class Store(Empowered):
             self._upgradeComplete = PendingEvent()
             d = self._upgradeService.addIterator(self._upgradeManager.upgradeEverything())
             def logUpgradeFailure(aFailure):
+                if aFailure.check(errors.ItemUpgradeError):
+                    log.err(aFailure.value.originalFailure, 'Item upgrade error')
                 log.err(aFailure, "upgrading %r failed" % (self,))
                 return aFailure
             d.addErrback(logUpgradeFailure)
@@ -1297,11 +1347,11 @@ class Store(Empowered):
 
     def findOrCreate(self, userItemClass, __ifnew=None, **attrs):
         """
-        Usage:
+        Usage::
 
             s.findOrCreate(userItemClass [, function] [, x=1, y=2, ...])
 
-        Example:
+        Example::
 
             class YourItemType(Item):
                 a = integer()
@@ -1555,11 +1605,11 @@ class Store(Empowered):
     def findFirst(self, tableClass, comparison=None,
                   offset=None, sort=None, default=None):
         """
-        Usage:
+        Usage::
 
             s.findFirst(tableClass [, query arguments except 'limit'])
 
-        Example:
+        Example::
 
             class YourItemType(Item):
                 a = integer()
@@ -1731,6 +1781,23 @@ class Store(Empowered):
     tablesCreatedThisTransaction = None
 
     def transact(self, f, *a, **k):
+        """
+        Execute C{f(*a, **k)} in the context of a database transaction.
+
+        Any changes made to this L{Store} by C{f} will be committed when C{f}
+        returns.  If C{f} raises an exception, those changes will be reverted
+        instead.
+
+        If a transaction is already in progress (in this thread - ie, if a
+        frame executing L{Store.transact} is already on the call stack), this
+        will B{not} start a nested transaction.  Changes will not be committed
+        until the existing transaction completes, and an exception raised by
+        C{f} will not revert changes made by C{f}.  You probably don't want to
+        ever call this if another transaction is in progress.
+
+        @return: Whatever C{f(*a, **kw)} returns.
+        @raise: Whatever C{f(*a, **kw)} raises, or a database exception.
+        """
         if self.transaction is not None:
             return f(*a, **k)
         if self.attachedToParent:
@@ -1895,7 +1962,8 @@ class Store(Empowered):
 
         @param tableClass: an Item subclass
 
-        @raises L{axiom.errors.ItemClassesOnly}: if an object other than a subclass of Item is passed.
+        @raises axiom.errors.ItemClassesOnly: if an object other than a
+            subclass of Item is passed.
 
         @return: a string
         """
@@ -2197,27 +2265,6 @@ class Store(Empowered):
         return default
 
 
-    def _normalizeSQL(self, sql):
-        # It turns out that "ATTACH DATABASE" *requires* string interpolation,
-        # since it syntactically does not support bind parameters.  It takes a
-        # string as a parameter though.  Considering that this assertion was
-        # never tripped before I don't feel too bad commenting it out, but I
-        # wish there were a way to preserve 'paranoid mode'
-
-        # assert "'" not in sql, "Strings are _NOT ALLOWED_"
-        if sql not in self.statementCache:
-            accum = []
-            lines = sql.split('\n')
-            for line in lines:
-                line = line.split('--')[0]         # remove comments
-                words = line.strip().split()
-                accum.extend(words)
-            normsql = ' '.join(accum)   # your SQL should never have any
-                                        # significant whitespace in it, right?
-            self.statementCache[sql] = normsql
-        return self.statementCache[sql]
-
-
     def querySchemaSQL(self, sql, args=()):
         sql = sql.replace("*DATABASE*", self.databaseName)
         return self.querySQL(sql, args)
@@ -2226,7 +2273,6 @@ class Store(Empowered):
     def querySQL(self, sql, args=()):
         """For use with SELECT (or SELECT-like PRAGMA) statements.
         """
-        sql = self._normalizeSQL(sql)
         if self.debug:
             result = timeinto(self.queryTimes, self._queryandfetch, sql, args)
         else:
@@ -2266,7 +2312,6 @@ class Store(Empowered):
 
 
     def _execSQL(self, sql, args):
-        sql = self._normalizeSQL(sql)
         if self.debug:
             rows = timeinto(self.execTimes, self._queryandfetch, sql, args)
         else:
